@@ -2,6 +2,9 @@ from fastapi import FastAPI, File, status, Request, UploadFile, Form, HTTPExcept
 from unstructured_inference.inference.layout import process_data_with_model
 from unstructured_inference.models.base import UnknownModelException
 from typing import List
+import tempfile
+
+from unstructured_inference.models.yolox import yolox_local_inference
 
 app = FastAPI()
 
@@ -9,7 +12,7 @@ ALL_ELEMS = "_ALL"
 VALID_FILETYPES = ["pdf", "image"]
 
 
-@app.post("/layout/{filetype:path}")
+@app.post("/layout/detectron/{filetype:path}")
 async def layout_parsing(
     filetype: str,
     file: UploadFile = File(),
@@ -38,6 +41,21 @@ async def layout_parsing(
     return {"pages": pages_layout}
 
 
+@app.post("/layout/yolox/{filetype:path}")
+async def layout_parsing_yolox(
+    filetype: str,
+    request: Request,
+    file: List[UploadFile] = File(default=None),
+    force_ocr=Form(default=False),
+):
+
+    with tempfile.NamedTemporaryFile() as tmp_file:
+        tmp_file.write(file[0].file.read())
+        detections = yolox_local_inference(tmp_file.name, type=filetype, use_ocr=force_ocr)
+
+    return detections
+
+
 @app.get("/healthcheck", status_code=status.HTTP_200_OK)
 async def healthcheck(request: Request):
     return {"healthcheck": "HEALTHCHECK STATUS: EVERYTHING OK!"}
