@@ -5,7 +5,9 @@ import re
 import tempfile
 from typing import List, Optional, Tuple, Union, BinaryIO
 
-import layoutparser as lp
+from layoutparser.io.pdf import load_pdf
+from layoutparser.elements.layout_elements import TextBlock
+from layoutparser.elements.layout import Layout
 from layoutparser.models.detectron2.layoutmodel import Detectron2LayoutModel
 import numpy as np
 from PIL import Image
@@ -13,13 +15,16 @@ from PIL import Image
 from unstructured_inference.logger import logger
 import unstructured_inference.models.tesseract as tesseract
 from unstructured_inference.models.base import get_model
+from unstructured_inference.models.unstructuredmodel import UnstructuredModel
+import cv2
 
 
 @dataclass
 class LayoutElement:
     type: str
-    # NOTE(robinson) - Coordinates are reported starting from the upper left and
-    # proceeding clockwise
+    # NOTE(robinson) - The list contain two elements, each a tuple
+    # in format (x1,y1), the first the upper left corner and the second
+    # the right bottom corner
     coordinates: List[Tuple[float, float]]
     text: Optional[str] = None
 
@@ -29,18 +34,30 @@ class LayoutElement:
     def to_dict(self):
         return self.__dict__
 
+    def get_width(self):
+        # NOTE(benjamin) i.e: y2-y1
+        return self.coordinates[1][0] - self.coordinates[0][0]
+
+    def get_height(self):
+        # NOTE(benjamin) i.e: x2-x1
+        return self.coordinates[1][1] - self.coordinates[0][1]
+
 
 class DocumentLayout:
     """Class for handling documents that are saved as .pdf files. For .pdf files, a
     document image analysis (DIA) model detects the layout of the page prior to extracting
     element."""
 
-    def __init__(self):
-        self._pages = None
+    def __init__(self, pages=None):
+        self._pages = pages
 
     def __str__(self) -> str:
         return "\n\n".join([str(page) for page in self.pages])
 
+    def to_string(self):
+        # Temporary method, this should replace __str__
+        return "\n\n".join([element.to_string() for element in self.pages])
+
     @property
     def pages(self) -> List[PageLayout]:
         """Gets all elements from pages in sequential order."""
@@ -61,7 +78,8 @@ class DocumentLayout:
         # a locally instantiated model or an API. Maybe even just a callable that accepts an
         # image and returns a dict, or something.
         logger.info(f"Reading PDF for file: {filename} ...")
-        layouts, images = lp.load_pdf(filename, load_images=True)
+        layouts, images = load_pdf(filename, load_images=True)
+        layouts, images = load_pdf(filename, load_images=True)
         pages: List[PageLayout] = list()
         for i, layout in enumerate(layouts):
             image = images[i]
@@ -87,6 +105,69 @@ class DocumentLayout:
         page.get_elements()
         return cls.from_pages([page])
 
+    def parse_elements(self, pdf_filename, DPI=500):
+        """
+        Fill the text of the document from embedded file
+        """
+        with tempfile.TemporaryDirectory() as tmp_folder:
+
+            for n_page, page in enumerate(self._pages):
+                new_layout = []
+                for n_element, element in enumerate(page.layout):
+
+                    (upper_left_x, upper_left_y) = element.coordinates[0]
+                    dest_file = os.path.join(tmp_folder, f"{n_page}-{n_element}.txt")
+
+                    cmd = (
+                        f"pdftotext -r {DPI} -x {int(upper_left_x)} -y {int(upper_left_y)} "
+                        + f"-W {int(element.get_width())} -H {int(element.get_height())} "
+                        + f"-f {page.number} -l {page.number} {pdf_filename} {dest_file}"
+                    )
+
+                    exit = os.system(cmd)
+
+                    if exit == 0:
+                        with open(dest_file) as file:
+                            content = file.read()
+                        element.text = content
+                        new_layout.append(element)
+                    else:
+                        continue
+                new_page = PageLayout(number=page.number, image=None, layout=new_layout)
+                self._pages[n_page] = new_page
+
+    def parse_image_elements(self, filename, num, DPI=500):
+        """
+        Fill the text of the document from OCR
+        """
+        with tempfile.TemporaryDirectory() as tmp_folder:
+            n_page = num
+            page = self._pages[n_page]
+
+            new_layout = []
+            for n_element, element in enumerate(page.layout):
+
+                (upper_left_x, upper_left_y) = element.coordinates[0]
+                upper_left_x = int(upper_left_x)
+                upper_left_y = int(upper_left_y)
+                width = upper_left_x + int(element.get_width())
+                height = upper_left_y + int(element.get_height())
+                dest_file = os.path.join(tmp_folder, f"{n_page}-{n_element}.jpg")
+
+                image = cv2.imread(filename)
+                patch = image[upper_left_y:height, upper_left_x:width]
+                cv2.imwrite(dest_file, patch)
+                # Enabling this makes test_load_agent fails
+                if not tesseract.ocr_agent:
+                    tesseract.load_agent()
+                text = tesseract.ocr_agent.detect(patch)
+
+                element.text = text
+                new_layout.append(element)
+
+            new_page = PageLayout(number=page.number, image=None, layout=new_layout)
+            self._pages[n_page] = new_page
+
 
 class PageLayout:
     """Class for an individual PDF page."""
@@ -95,8 +176,8 @@ class PageLayout:
         self,
         number: int,
         image: Image,
-        layout: Optional[lp.Layout],
-        model: Optional[Detectron2LayoutModel] = None,
+        layout: Layout,
+        model: Optional[UnstructuredModel] = None,
     ):
         self.image = image
         self.image_array: Union[np.ndarray, None] = None
@@ -108,8 +189,12 @@ class PageLayout:
     def __str__(self):
         return "\n\n".join([str(element) for element in self.elements])
 
+    def to_string(self):
+        """Temporary method, should replace __str__"""
+        return "\n\n".join([str(element) for element in self.layout])
+
     def get_elements(self, inplace=True) -> Optional[List[LayoutElement]]:
-        """Uses a layoutparser model to detect the elements on the page."""
+        """Uses specified model to detect the elements on the page."""
         logger.info("Detecting page elements ...")
         if self.model is None:
             self.model = get_model()
@@ -117,7 +202,7 @@ class PageLayout:
         elements = list()
         # NOTE(mrobinson) - We'll want make this model inference step some kind of
         # remote call in the future.
-        image_layout = self.model.detect(self.image)
+        image_layout = self.model(self.image)
         # NOTE(robinson) - This orders the page from top to bottom. We'll need more
         # sophisticated ordering logic for more complicated layouts.
         image_layout.sort(key=lambda element: element.coordinates[1], inplace=True)
@@ -139,7 +224,7 @@ class PageLayout:
             return None
         return elements
 
-    def interpret_text_block(self, text_block: lp.TextBlock) -> str:
+    def interpret_text_block(self, text_block: TextBlock) -> str:
         """Interprets the text in a TextBlock."""
         # NOTE(robinson) - If the text attribute is None, that means the PDF isn't
         # already OCR'd and we have to send the snippet out for OCRing.
@@ -149,7 +234,7 @@ class PageLayout:
             out_text = text_block.text
         return out_text
 
-    def ocr(self, text_block: lp.TextBlock) -> str:
+    def ocr(self, text_block: TextBlock) -> str:
         """Runs a cropped text block image through and OCR agent."""
         logger.debug("Running OCR on text block ...")
         tesseract.load_agent()
